###########
# 1. Extract temperature data for a period of 10 years and for a specific region of the Iberian Peninsula
# 2. Convert temperature from Kelvin to Degrees
# 3. Extract dew point temperature for the same period and region
# 4. Convert temperature from Kelvin to Degrees
# 5. Compute relative humidity
# 6. Extract precipitation for the same period and region
# 7. Convert m to mm
# 8. Create the binary indicator
# 9. Use totaltimeexceedingthreshold to create summary value per year
#########

source("https://earth.bsc.es/gitlab/nperez/Flor/-/raw/master/CST_Load_devel_from_s2dv.R")
source("plot_map.R")

path_ERA5tas_CDS <- list(path = '/esarchive/recon/ecmwf/era5land/$STORE_FREQ$_mean/$VAR_NAME$_f1h/$VAR_NAME$_$YEAR$$MONTH$.nc')

year_in <- 2020
year_fi <- 2021
month_in <- 1
month_fi <- 12

sdates <- paste0(year_in:year_fi, '0101')

### TEMPERATURE (TAS)
# 1. Extract sample temperature
tas_obs <- CST_Load_s2dv(var = 'tas',
                            exp = NULL,
                            obs = list(path_ERA5tas_CDS),
                            sdates = sdates,
                            lonmax = 355, lonmin = 350,
                            latmax = 45, latmin = 43,
                            storefreq = 'monthly',
                            leadtimemin = month_in, 
                            leadtimemax = month_fi,
                            output = "lonlat")

# 2. Convert temperature from Kelvin to Degrees
tas_obs$data <- tas_obs$data-273.15

### DEW POINT TEMPERATURE (TDPS)
# 3. Extract sample dew point temperature
tdps_obs <- CST_Load_s2dv(var = 'tdps',
                         exp = NULL,
                         obs = list(path_ERA5tas_CDS),
                         sdates = sdates,
                         lonmax = 355, lonmin = 350,
                         latmax = 45, latmin = 43,
                         storefreq = 'monthly',
                         leadtimemin = month_in, 
                         leadtimemax = month_fi,
                         output = "lonlat")


# 4. Convert from Kelvin to Degrees
tdps_obs$data <- tdps_obs$data-273.15

# 5. Compute rel humidity
rel_hum <- tdps_obs
rel_hum$data <- 100*(exp((17.625*tdps_obs$data)/(243.04+tdps_obs$data)) / exp((17.625*tas_obs$data) / (243.04+tas_obs$data)))

#### PRECIPITATION (PRLR)
# 6. Extract sample precipitation
pr_obs <- CST_Load_s2dv(var = 'prlr',
                         exp = NULL,
                         obs = list(path_ERA5tas_CDS),
                         sdates = sdates,
                         lonmax = 355, lonmin = 350,
                         latmax = 45, latmin = 43,
                         storefreq = 'monthly',
                         leadtimemin = month_in, 
                         leadtimemax = month_fi,
                         output = "lonlat")

# 7. Convert m/month to mm/moth
# Precipitation is downloaded and transformed as a flux of meters per second per
# month, to transform it to mm/month it is necessary to multiply it by:
# - total number of seconds in one hour: 3600
# - total number of hours in a day: 24
# - average number of days in a month: 30.41
# - change from meter to mm: 1000
pr_obs$data <- pr_obs$data*3600*24*30.41*1000

#### INDICATOR
# 8. Create the binary indicator
# i) Create individual suitability per variable for checking purposes
t1 <- r1 <- p1 <- pr_obs
t1$data <- ifelse(tas_obs$data >= 14.5 & tas_obs$data <= 33,1,0)
# tas_obs$data[1,1,1,6,,][15:19, 15:20]
# t1$data[1,1,1,6,,][15:19, 15:20]

r1$data <- ifelse(rel_hum$data >= 60,1,0)
# rel_hum$data[1,1,1,6,,][15:19, 15:20]
# r1$data[1,1,1,6,,][15:19, 15:20]

p1$data <- ifelse(pr_obs$data >= 80,1,0)
# pr_obs$data[1,1,1,6,,][15:19, 15:20]
# p1$data[1,1,1,6,,][15:19, 15:20]

# ii) Create climate suitability
suit_ind <- pr_obs
suit_ind$data <- ifelse(tas_obs$data >= 14.5 & tas_obs$data <= 33 & rel_hum$data >= 60 & pr_obs$data >= 80,1,0)
# suit_ind$data[1,1,1,6,,][15:19, 15:20]

# 9. Use CSIndicators::TotalTimeExceedingThreshold() to compute indicator
test <- CSIndicators::CST_TotalTimeExceedingThreshold(suit_ind, threshold=0.5)#, time_dim="sdate")

dim(test$data)
test$data[1,1,1,,][15:19, 15:20]